==> swc-1.3.37.tar.gz: swc-1.3.37/crates/swc_ecma_transforms_base/src/perf.rs <==
use swc_common::util::move_map::MoveMap;
#[cfg(feature = "concurrent")]
use swc_common::{errors::HANDLER, GLOBALS};
use swc_ecma_ast::*;
pub use swc_ecma_utils::parallel::*;
use swc_ecma_visit::{Fold, FoldWith, Visit, VisitMut, VisitMutWith, VisitWith};

#[cfg(feature = "concurrent")]
use crate::helpers::HELPERS;

pub trait Check: Visit + Default {
    fn should_handle(&self) -> bool;
}

pub fn should_work<C, T>(n: &T) -> bool
where
    C: Check,
    T: VisitWith<C>,
{
    let mut checker = C::default();
    n.visit_with(&mut checker);
    checker.should_handle()
}

pub trait ParExplode: Parallel {
    /// Invoked after visiting each statements.
    ///
    /// Implementor should not delete/prepend to `stmts`.
    fn after_one_stmt(&mut self, stmts: &mut Vec<Stmt>);

    /// Invoked after visiting each statements.
    ///
    /// Implementor should not delete/prepend to `stmts`.
    fn after_one_module_item(&mut self, stmts: &mut Vec<ModuleItem>);
}

pub trait ParVisit: Visit + Parallel {
    fn visit_par<N>(&mut self, threshold: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParVisit for T
where
    T: Visit + Parallel,
{
    fn visit_par<N>(&mut self, threshold: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>,
    {
        if nodes.len() >= threshold || option_env!("SWC_FORCE_CONCURRENT") == Some("1") {
            GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        use rayon::prelude::*;

                        let visitor = nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            node.visit_with(&mut visitor);

                                            visitor
                                        })
                                    })
                                })
                            })
                            .reduce(
                                || Parallel::create(&*self),
                                |mut a, b| {
                                    Parallel::merge(&mut a, b);

                                    a
                                },
                            );

                        Parallel::merge(self, visitor);
                    })
                })
            });

            return;
        }

        for n in nodes {
            n.visit_with(self);
        }
    }
}

pub trait ParVisitMut: VisitMut + Parallel {
    fn visit_mut_par<N>(&mut self, threshold: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParVisitMut for T
where
    T: VisitMut + Parallel,
{
    fn visit_mut_par<N>(&mut self, threshold: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>,
    {
        if nodes.len() >= threshold || option_env!("SWC_FORCE_CONCURRENT") == Some("1") {
            GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        use rayon::prelude::*;

                        let visitor = nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            node.visit_mut_with(&mut visitor);

                                            visitor
                                        })
                                    })
                                })
                            })
                            .reduce(
                                || Parallel::create(&*self),
                                |mut a, b| {
                                    Parallel::merge(&mut a, b);

                                    a
                                },
                            );

                        Parallel::merge(self, visitor);
                    })
                })
            });

            return;
        }

        for n in nodes {
            n.visit_mut_with(self);
        }
    }
}

pub trait ParFold: Fold + Parallel {
    fn fold_par<N>(&mut self, threshold: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParFold for T
where
    T: Fold + Parallel,
{
    fn fold_par<N>(&mut self, threshold: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>,
    {
        if nodes.len() >= threshold || option_env!("SWC_FORCE_CONCURRENT") == Some("1") {
            use rayon::prelude::*;

            let (visitor, nodes) = GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            let node = node.fold_with(&mut visitor);

                                            (visitor, node)
                                        })
                                    })
                                })
                            })
                            .fold(
                                || (Parallel::create(&*self), vec![]),
                                |mut a, b| {
                                    Parallel::merge(&mut a.0, b.0);

                                    a.1.push(b.1);

                                    a
                                },
                            )
                            .reduce(
                                || (Parallel::create(&*self), vec![]),
                                |mut a, b| {
                                    Parallel::merge(&mut a.0, b.0);

                                    a.1.extend(b.1);

                                    a
                                },
                            )
                    })
                })
            });

            Parallel::merge(self, visitor);

            return nodes;
        }

        nodes.move_map(|n| n.fold_with(self))
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParVisit for T
where
    T: Visit + Parallel,
{
    fn visit_par<N>(&mut self, _: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>,
    {
        for n in nodes {
            n.visit_with(self);
        }
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParVisitMut for T
where
    T: VisitMut + Parallel,
{
    fn visit_mut_par<N>(&mut self, _: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>,
    {
        for n in nodes {
            n.visit_mut_with(self);
        }
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParFold for T
where
    T: Fold + Parallel,
{
    fn fold_par<N>(&mut self, _: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>,
    {
        nodes.move_map(|n| n.fold_with(self))
    }
}

==> swc-1.3.97.tar.gz: swc-1.3.97/crates/swc_ecma_transforms_base/src/perf.rs <==
use swc_common::util::move_map::MoveMap;
#[cfg(feature = "concurrent")]
use swc_common::{errors::HANDLER, GLOBALS};
use swc_ecma_ast::*;
pub use swc_ecma_utils::parallel::*;
use swc_ecma_visit::{Fold, FoldWith, Visit, VisitMut, VisitMutWith, VisitWith};

#[cfg(feature = "concurrent")]
use crate::helpers::HELPERS;

pub trait Check: Visit + Default {
    fn should_handle(&self) -> bool;
}

pub fn should_work<C, T>(n: &T) -> bool
where
    C: Check,
    T: VisitWith<C>,
{
    let mut checker = C::default();
    n.visit_with(&mut checker);
    checker.should_handle()
}

pub trait ParExplode: Parallel {
    /// Invoked after visiting each statements.
    ///
    /// Implementor should not delete/prepend to `stmts`.
    fn after_one_stmt(&mut self, stmts: &mut Vec<Stmt>);

    /// Invoked after visiting each statements.
    ///
    /// Implementor should not delete/prepend to `stmts`.
    fn after_one_module_item(&mut self, stmts: &mut Vec<ModuleItem>);
}

pub trait ParVisit: Visit + Parallel {
    fn visit_par<N>(&mut self, threshold: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParVisit for T
where
    T: Visit + Parallel,
{
    fn visit_par<N>(&mut self, threshold: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>,
    {
        if nodes.len() >= threshold {
            GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        use rayon::prelude::*;

                        let visitor = nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            node.visit_with(&mut visitor);

                                            visitor
                                        })
                                    })
                                })
                            })
                            .reduce(
                                || Parallel::create(&*self),
                                |mut a, b| {
                                    Parallel::merge(&mut a, b);

                                    a
                                },
                            );

                        Parallel::merge(self, visitor);
                    })
                })
            });

            return;
        }

        for n in nodes {
            n.visit_with(self);
        }
    }
}

pub trait ParVisitMut: VisitMut + Parallel {
    fn visit_mut_par<N>(&mut self, threshold: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParVisitMut for T
where
    T: VisitMut + Parallel,
{
    fn visit_mut_par<N>(&mut self, threshold: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>,
    {
        if nodes.len() >= threshold {
            GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        use rayon::prelude::*;

                        let visitor = nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            node.visit_mut_with(&mut visitor);

                                            visitor
                                        })
                                    })
                                })
                            })
                            .reduce(
                                || Parallel::create(&*self),
                                |mut a, b| {
                                    Parallel::merge(&mut a, b);

                                    a
                                },
                            );

                        Parallel::merge(self, visitor);
                    })
                })
            });

            return;
        }

        for n in nodes {
            n.visit_mut_with(self);
        }
    }
}

pub trait ParFold: Fold + Parallel {
    fn fold_par<N>(&mut self, threshold: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>;
}

#[cfg(feature = "concurrent")]
impl<T> ParFold for T
where
    T: Fold + Parallel,
{
    fn fold_par<N>(&mut self, threshold: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>,
    {
        if nodes.len() >= threshold {
            use rayon::prelude::*;

            let (visitor, nodes) = GLOBALS.with(|globals| {
                HELPERS.with(|helpers| {
                    HANDLER.with(|handler| {
                        nodes
                            .into_par_iter()
                            .map(|node| {
                                GLOBALS.set(globals, || {
                                    HELPERS.set(helpers, || {
                                        HANDLER.set(handler, || {
                                            let mut visitor = Parallel::create(&*self);
                                            let node = node.fold_with(&mut visitor);

                                            (visitor, node)
                                        })
                                    })
                                })
                            })
                            .fold(
                                || (Parallel::create(&*self), vec![]),
                                |mut a, b| {
                                    Parallel::merge(&mut a.0, b.0);

                                    a.1.push(b.1);

                                    a
                                },
                            )
                            .reduce(
                                || (Parallel::create(&*self), vec![]),
                                |mut a, b| {
                                    Parallel::merge(&mut a.0, b.0);

                                    a.1.extend(b.1);

                                    a
                                },
                            )
                    })
                })
            });

            Parallel::merge(self, visitor);

            return nodes;
        }

        nodes.move_map(|n| n.fold_with(self))
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParVisit for T
where
    T: Visit + Parallel,
{
    fn visit_par<N>(&mut self, _: usize, nodes: &[N])
    where
        N: Send + Sync + VisitWith<Self>,
    {
        for n in nodes {
            n.visit_with(self);
        }
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParVisitMut for T
where
    T: VisitMut + Parallel,
{
    fn visit_mut_par<N>(&mut self, _: usize, nodes: &mut [N])
    where
        N: Send + Sync + VisitMutWith<Self>,
    {
        for n in nodes {
            n.visit_mut_with(self);
        }
    }
}

#[cfg(not(feature = "concurrent"))]
impl<T> ParFold for T
where
    T: Fold + Parallel,
{
    fn fold_par<N>(&mut self, _: usize, nodes: Vec<N>) -> Vec<N>
    where
        N: Send + Sync + FoldWith<Self>,
    {
        nodes.move_map(|n| n.fold_with(self))
    }
}
